#!/bin/bash

# 设置颜色输出
GREEN='\033[0;32m'
BLUE='\033[0;34m'
NC='\033[0m' # No Color

# 确保脚本使用正确
if [ $# -lt 1 ]; then
  echo "用法: $0 <程序路径> [目标目录]"
  echo "例如: $0 ./myapp ./libs"
  exit 1
fi

# 参数处理
EXECUTABLE="$1"
if [ ! -f "$EXECUTABLE" ]; then
  echo "错误: 文件 $EXECUTABLE 不存在!"
  exit 1
fi

# 目标目录
TARGET_DIR="./lib"
if [ ! -z "$2" ]; then
  TARGET_DIR="$2"
fi

# 创建目标目录
mkdir -p "$TARGET_DIR"
echo -e "${GREEN}将收集 $EXECUTABLE 的所有依赖库到 $TARGET_DIR${NC}"

# 已处理的库集合
declare -A PROCESSED_LIBS

# 递归查找并拷贝库的函数
copy_dependencies() {
  local binary="$1"
  local indent="$2"
  
  # 对于ELF文件才进行处理
  if ! file "$binary" | grep -q "ELF"; then
    return
  fi
  
  echo -e "${indent}${BLUE}分析 $(basename $binary) 的依赖...${NC}"
  
  # 获取所有动态库依赖
  ldd "$binary" | grep "=>" | while read line; do
    # 提取库路径
    local lib=$(echo $line | awk '{print $3}')
    
    # 跳过不存在的库和已经处理过的库
    if [ -z "$lib" ] || [ "$lib" == "not" ] || [ ! -f "$lib" ]; then
      continue
    fi
    
    # 库的基本名称
    local basename=$(basename "$lib")
    
    # 如果库已处理过，则跳过
    if [ ${PROCESSED_LIBS[$lib]+_} ]; then
      echo -e "${indent}  已处理: $basename"
      continue
    fi
    
    # 标记为已处理
    PROCESSED_LIBS[$lib]=1
    
    # 复制库文件
    if [ ! -f "$TARGET_DIR/$basename" ]; then
      echo -e "${indent}  ${GREEN}复制: $basename${NC}"
      cp -L "$lib" "$TARGET_DIR/"
      
      # 递归处理这个库的依赖
      copy_dependencies "$lib" "${indent}  "
    else
      echo -e "${indent}  已存在: $basename"
    fi
  done
}

# 开始递归处理
copy_dependencies "$EXECUTABLE" ""

# 统计并报告
echo -e "\n${GREEN}完成!${NC}"
echo -e "总共复制了 ${#PROCESSED_LIBS[@]} 个库到 $TARGET_DIR/"
echo -e "库列表:"
ls -la "$TARGET_DIR"

# 提示设置RPATH或LD_LIBRARY_PATH
echo -e "\n${BLUE}提示:${NC}"
echo -e "要运行您的程序，您可以:"
echo -e "1. 设置LD_LIBRARY_PATH: export LD_LIBRARY_PATH=$TARGET_DIR:\$LD_LIBRARY_PATH"
echo -e "2. 或使用patchelf设置RPATH: patchelf --set-rpath '\$ORIGIN/$TARGET_DIR' $EXECUTABLE"
echo -e "3. 或创建一个启动脚本设置LD_LIBRARY_PATH"